@@ -2,9 +2,11 @@
 #![warn(missing_docs)]
 #![warn(rustdoc::missing_crate_level_docs)]
 #![feature(hash_set_entry)]
+#![feature(once_cell)]
 
 mod gatherer;
-pub use gatherer::{Gatherer, GathererBuilder, ProcessFn};
+pub use gatherer::{Gatherer, GathererBuilder, GathererHandle, ProcessFn, ProcessMessage};
+pub(crate) use gatherer::GathererInner;
 
 mod messages;
 pub use messages::{DirectoryGatherMessage, InventoryEntryMessage};
@@ -13,17 +15,16 @@ mod objectpath;
 pub use objectpath::ObjectPath;
 
 mod internednames;
-use internednames::{InternedName, InternedNames};
+pub use internednames::{InternedName, InternedNames};
 
 mod dirhandle;
-use dirhandle::Dir;
-pub use dirhandle::used_handles;
+pub use dirhandle::{used_handles, Dir};
 pub use openat_ct as openat;
 
 /// An user defined processing function can return any kind of error, this needs to be boxed
 /// and dyn. Since error handling is expected to be the slow path, having the allocation and
 /// vtable here shouldn't be an performance issue.
-pub type DynError = Box<dyn std::error::Error + Send>;
+pub type DynError = Box<dyn std::error::Error + Send + Sync + 'static>;
 /// Typedef for the DynError result.
 pub type DynResult<T> = std::result::Result<T, DynError>;
 
@@ -34,6 +35,8 @@ mod test {
     use std::io::Write;
     use std::sync::Once;
 
+    #[allow(unused_imports)]
+    pub use log::{debug, error, info, trace, warn};
     use env_logger;
 
     pub fn init_env_logging() {
@@ -64,4 +67,10 @@ mod test {
                 .unwrap();
         });
     }
+
+    #[test]
+    fn smoke() {
+        crate::test::init_env_logging();
+        info!("logging works");
+    }
 }
